@@ -30,6 +30,10 @@
 #include "xt_pknock.h"
 #include "compat_xtables.h"
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 2, 0)
+#define timer_delete(X) del_timer(X)
+#endif
+
 enum status {
 	ST_INIT = 1,
 	ST_MATCHING,
@@ -296,7 +300,7 @@ static const struct proc_ops pknock_proc_ops = {
 static void update_rule_gc_timer(struct xt_pknock_rule *rule)
 {
 	if (timer_pending(&rule->timer))
-		del_timer(&rule->timer);
+		timer_delete(&rule->timer);
 	rule->timer.expires = jiffies + msecs_to_jiffies(gc_expir_time);
 	add_timer(&rule->timer);
 }
@@ -517,7 +521,7 @@ remove_rule(struct xt_pknock_mtinfo *info)
 		remove_proc_entry(info->rule_name, pde);
 	pr_debug("(D) rule deleted: %s.\n", rule->rule_name);
 	if (timer_pending(&rule->timer))
-		del_timer(&rule->timer);
+		timer_delete(&rule->timer);
 	list_del(&rule->head);
 	kfree(rule->peer_head);
 	kfree(rule);
@@ -718,7 +722,6 @@ has_secret(const unsigned char *secret, unsigned int secret_len, uint32_t ipsrc,
 	unsigned int hexa_size;
 	int ret;
 	bool fret = false;
-	uint64_t x;
 	unsigned int epoch_min;
 	/* Concurrent use fenced off by a caller which holds list_lock. */
 	struct shash_desc *shash = (void *)crypto.desc; // SHASH_DESC_ON_STACK part 2
@@ -733,9 +736,13 @@ has_secret(const unsigned char *secret, unsigned int secret_len, uint32_t ipsrc,
 	hexresult = kzalloc(hexa_size, GFP_ATOMIC);
 	if (hexresult == NULL)
 		return false;
-	x = ktime_get_seconds();
-	do_div(x, 60);
-	epoch_min = x;
+
+	/* Time needs to be in minutes relative to epoch. */
+	{
+		time64_t t = ktime_get_real_seconds();
+		do_div(t, 60);
+		epoch_min = t;
+	}
 
 	ret = crypto_shash_setkey(crypto.tfm, secret, secret_len);
 	if (ret != 0) {
